/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

/*
** Altered by Gordon Hollingworth to remove the error reporting
** channels, and so implement with the JBits XHWIF interface
*/

//
// Pport objects handle the parallel port data, status, and
// control pins.
//

// $Log: pport.cpp $
// Revision 1.2  1999/03/30 18:27:12  DEVB
// Released source into the public domain
//
// Revision 1.1  1999/03/29 21:57:52  DEVB
// Initial revision
//

#include <stdlib.h>
#include <conio.h>
#include <assert.h>
#include <strstrea.h>
#include <wtypes.h>
#include <stdio.h>

#include "dlportio.h"
#include "pport.h"



// begin methods and functions

//
// Here are some routines for handling primitive read/write of ports
//
#define MAX_STACK 1000
unsigned int stackSize=0;
unsigned char stack[MAX_STACK];
unsigned short adds[MAX_STACK];

void FlushStack()
{
	unsigned int point=0,i;

	while(point<stackSize)
	{
		for(i=point;adds[i]==adds[point]  && i<stackSize;i++); /*Number to send == i-point */
        DlPortWritePortBufferUchar(adds[point], &stack[point], i-point);
        point += i-point;
	}
	stackSize=0;
}
// get data byte from a given 16-bit I/O address
static unsigned char Inp(unsigned short address)
{
	if(stackSize>0) FlushStack();
	return DlPortReadPortUchar(address);
}


// send data byte to a given 16-bit I/O address
static void Outp(unsigned short address, unsigned char byte)
{
	if(stackSize<MAX_STACK)
	{
	   //printf("Adding address = %x, data = %x\n", address, byte);
	   adds[stackSize] = address;
	   stack[stackSize++] = byte;
	}
	else
	{
		FlushStack();
		//printf("Adding address = %x, data = %x\n", address, byte);
		adds[stackSize] = address;
		stack[stackSize++] = byte;
	}
}



//
// Here are some routines for handling bit fields
//

#define LOWFIELDMASK(lo,hi)	((1<<((hi)-(lo)+1))-1)
#define FIELDMASK(lo,hi)	(LOWFIELDMASK(lo,hi)<<(lo))

static const unsigned int minBitPos = 0;	// we only work with 8-bit bytes
static const unsigned int maxBitPos = 7;

// Check for bit position index which is too large
static bool CheckBitPos(unsigned int pos)
{
	if(pos>maxBitPos) // pos is unsigned, so no check against minBitPos
	{
		return false;
	}
	return true;
}

// Get the value stored in a bit field
static unsigned int GetField(unsigned int data,		// port data
							 unsigned int loPos,	// low bit position of field
							 unsigned int hiPos		// high bit position of field
							 )
{
	assert(loPos<=maxBitPos);
	assert(hiPos<=maxBitPos);
	assert(hiPos>=loPos);
	return (FIELDMASK(loPos,hiPos) & data) >> loPos;
}

// Set the value in a bit field and return the entire data value after the
// field is updated.
static unsigned int SetField(unsigned int data,		// port data
							 unsigned int loPos,	// low bit position of field
							 unsigned int hiPos,	// high bit position of field
							 unsigned int newData	// new data for field
							 )
{
	assert(loPos<=maxBitPos);
	assert(hiPos<=maxBitPos);
	assert(hiPos>=loPos);
	return (data & ~FIELDMASK(loPos,hiPos)) |
		((newData & FIELDMASK(0,hiPos-loPos))<<loPos);
}


// Global parameters for the parallel port
static const unsigned int defaultPortNum = 1;	// default parallel port #
static const unsigned int minPortNum = 1;		// minimum parallel port #
static const unsigned int maxPortNum = 4;		// maximum parallel port #
static const unsigned int dataOffset    = 0;	// data register address offset
static const unsigned int statusOffset  = 1;	// status register address offset
static const unsigned int controlOffset = 2;	// control register address offset


// Constructor for a parallel port object.  Inversion masks are used to
// correct the effects of the various inverters in the PC and XS Board.
Pport::Pport(
			 unsigned int n,			// parallel port number
			 unsigned int dInvMask,		// inversion mask for data output bits
			 unsigned int sInvMask,		// inversion mask for status input bits
			 unsigned int cInvMask		// inversion mask for control output bits
			 )
{
	SetNum(n);						// set port read, write addresses
	SetDataInvMask(dInvMask);		// set read, write inversion masks
	SetStatusInvMask(sInvMask);		// set read, write inversion masks
	SetControlInvMask(cInvMask);	// set read, write inversion masks
}

// Destructor for parallel port object
Pport::~Pport(void)
{
	; // nothing to do
}


// Assignment operator for parallel port objects
Pport& Pport::operator=(Pport& src)
{
	num            = src.num;
	address        = src.address;
	dataInvMask    = src.dataInvMask;
	statusInvMask  = src.statusInvMask;
	controlInvMask = src.controlInvMask;
	return *this;
}

// Sets up a port object for a given parallel port n
void Pport::SetNum(unsigned int n)
{
	num = n;
	FindIOAddress(); // finds the I/O address for the given parallel port #
	assert(address!=0);
}

// Gets the parallel port number for a parallel port object
unsigned int Pport::GetNum(void) const
{
	return num;
}

// 	Determines I/O address for the given parallel port # (1-4).
void Pport::FindIOAddress(void)
{
	unsigned short pportAddresses[maxPortNum];
	
#if 1
    // default the printer port addresses to their most common values.
	// we need to change this to something like the code shown below.
    pportAddresses[0] = 0x378;
    pportAddresses[1] = 0x278;
    pportAddresses[2] = 0x3BC;
    pportAddresses[3] = 0x000;
#else
	// get the parallel port addresses stored in memory
	// starting at memory address 0x408 and move them to the address array.
	// non-existent printer ports have addresses of 0.
	// This code doesn't work in Win95 or winNT because they don't allow
	// us to access memory like this in a non-VXD.
	asm{
		mov ax,0x0
			mov es,ax
			mov bx,es:0x408
			mov [pportAddresses+0],bx
			mov bx,es:0x40a
			mov [pportAddresses+2],bx
			mov bx,es:0x40c
			mov [pportAddresses+4],bx
			mov bx,es:0x40e
			mov [pportAddresses+6],bx
	}
#endif
	address = pportAddresses[num-1];
}

// Set the data register inversion mask
void Pport::SetDataInvMask(unsigned int invMask)
{
	dataInvMask = invMask;
}

// Output a value on the designated data pins of the parallel port
void Pport::SetData(
					unsigned int v,		// value to output
					unsigned int loPos,	// low bit position of field
					unsigned int hiPos	// high bit position of field
					)
{
	if(CheckBitPos(loPos) && CheckBitPos(hiPos))
	{
		// get the current value output on the data pins
		unsigned int d = GetData(minBitPos,maxBitPos);
		// over-write the specified bit field with the new value
		d = SetField(d,loPos,hiPos,v);
		// apply the inversion mask
		d ^= dataInvMask;
		// output the new data onto the parallel port data pins
		Outp(address+dataOffset, d);
	}
}

// Return the current values on the designated data pins of the parallel port
unsigned int Pport::GetData(
					unsigned int loPos,	// low bit position of field
					unsigned int hiPos	// high bit position of field
					) const
{
	if(CheckBitPos(loPos) && CheckBitPos(hiPos))
	{
		// first get the values on the parallel port data pins
		unsigned int d = Inp(address+dataOffset);
		// then account for any further inversions
		d ^= dataInvMask;
		return GetField(d, loPos, hiPos);
	}
	return 0;
}

// Set the status register inversion mask
void Pport::SetStatusInvMask(unsigned int invMask)
{
	statusInvMask = invMask;
}

// Return the current values on the designated status pins of the parallel port 
unsigned int Pport::GetStatus(
					unsigned int loPos,	// low bit position of field
					unsigned int hiPos	// high bit position of field
					) const
{
	if(CheckBitPos(loPos) && CheckBitPos(hiPos))
	{
		// first get the values on the status pins
		unsigned int d = Inp(address+statusOffset);
		// then account for any further inversions
		d = d ^ statusInvMask;
		return GetField(d, loPos, hiPos);
	}
	return 0;
}

// Set the control register inversion mask
void Pport::SetControlInvMask(unsigned int invMask)
{
	controlInvMask = invMask;
}

unsigned int currentControl;
unsigned int firstTime=-1;

// Output a value on the designated control pins of the parallel port
void Pport::SetControl(
				unsigned int v,
				unsigned int loPos,	// low bit position of field
				unsigned int hiPos	// high bit position of field
				)
{
	if(CheckBitPos(loPos) && CheckBitPos(hiPos))
	{
		unsigned int d;
		// get the current value output on the control pins
        if(firstTime==-1)
		{
			currentControl = GetControl(minBitPos,maxBitPos);
			firstTime=0;
		}
		d=currentControl;
		// over-write the specified bit field with the new value
		d = SetField(d,loPos,hiPos,v);
		// apply the inversion mask
		d ^= controlInvMask;
		// output the new data onto the parallel port control pins
		Outp(address+controlOffset, d);
		currentControl = d^controlInvMask;
	}
}

// Return the current values on the designated control pins of the parallel port
unsigned int Pport::GetControl(
					unsigned int loPos,	// low bit position of field
					unsigned int hiPos	// high bit position of field
				   ) const
{
	if(CheckBitPos(loPos) && CheckBitPos(hiPos))
	{
		// first get the values on the parallel port control pins
		unsigned int d = Inp(address+controlOffset);
		// then account for any further inversions
		d ^= controlInvMask;
		return GetField(d, loPos, hiPos);
	}
	return 0;
}

